@@ -28,7 +28,7 @@
 
     :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
     :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
-    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :copyright: Copyright 2016-2018 by the mss team, see AUTHORS.
     :license: APACHE-2.0, see LICENSE for details.
 
     Licensed under the Apache License, Version 2.0 (the "License");
@@ -59,8 +59,6 @@ from mslib.utils import get_distance, find_location
 # local application imports
 from mslib.msui import flighttrack as ft
 
-MOVE, DELETE, INSERT = list(range(3))
-
 
 def distance_point_linesegment(p, l1, l2):
     """Computes the distance between a point p and a line segment given by its
@@ -172,9 +170,9 @@ class WaypointsPath(mpath.Path):
             for i, _ in enumerate(wps[1:]):
                 pathdata.append((Path.LINETO, self.transform_waypoint(wps, i + 1)))
 
-        self.codes, self.vertices = list(zip(*pathdata))
-        self.codes = np.array(self.codes, dtype=np.uint8)
-        self.vertices = np.array(self.vertices)
+        codes, vertices = list(zip(*pathdata))
+        self.codes = np.array(codes, dtype=np.uint8)
+        self.vertices = np.array(vertices)
 
 
 #
@@ -194,7 +192,7 @@ class PathV(WaypointsPath):
                         points.
         """
         self.numintpoints = kwargs.pop("numintpoints")
-        WaypointsPath.__init__(self, *args, **kwargs)
+        super(PathV, self).__init__(*args, **kwargs)
 
     def update_from_WaypointsTableModel(self, wps_model):
         """Extended version of the corresponding WaypointsPath method.
@@ -257,7 +255,7 @@ class PathH(WaypointsPath):
            vertex cooredinates between lat/lon and projection coordinates.
         """
         self.map = kwargs.pop("map")
-        WaypointsPath.__init__(self, *args, **kwargs)
+        super(PathH, self).__init__(*args, **kwargs)
 
     def transform_waypoint(self, wps_list, index):
         """Transform lon/lat to projection coordinates.
@@ -292,9 +290,9 @@ class PathH_GC(PathH):
             pathdata = [(Path.MOVETO, self.transform_waypoint(wps, 0))]
             for i in range(len(wps[1:])):
                 pathdata.append((Path.LINETO, self.transform_waypoint(wps, i + 1)))
-        self.wp_codes, self.wp_vertices = list(zip(*pathdata))
-        self.wp_codes = np.array(self.wp_codes, dtype=np.uint8)
-        self.wp_vertices = np.array(self.wp_vertices)
+        wp_codes, wp_vertices = list(zip(*pathdata))
+        self.wp_codes = np.array(wp_codes, dtype=np.uint8)
+        self.wp_vertices = np.array(wp_vertices)
 
         # Coordinates of intermediate great circle points.
         lons, lats = list(zip(*[(wp.lon, wp.lat) for wp in wps]))
@@ -304,9 +302,9 @@ class PathH_GC(PathH):
             pathdata = [(Path.MOVETO, (x[0], y[0]))]
             for i in range(len(x[1:])):
                 pathdata.append((Path.LINETO, (x[i + 1], y[i + 1])))
-        self.codes, self.vertices = list(zip(*pathdata))
-        self.codes = np.array(self.codes, dtype=np.uint8)
-        self.vertices = np.array(self.vertices)
+        codes, vertices = list(zip(*pathdata))
+        self.codes = np.array(codes, dtype=np.uint8)
+        self.vertices = np.array(vertices)
 
     def index_of_closest_segment(self, x, y, eps=5):
         """Find the index of the edge closest to the specified point at x,y.
@@ -352,7 +350,7 @@ class PathInteractor(object):
     """
 
     showverts = True  # show the vertices of the path patch
-    epsilon = 5  # max pixel distance to count as a vertex hit when
+    epsilon = 12  # max pixel distance to count as a vertex hit when
 
     # picking points
 
@@ -378,7 +376,6 @@ class PathInteractor(object):
         self.waypoints_model = None
         self.background = None
         self._ind = None  # the active vertex
-        self.editmode = MOVE
 
         # Create a PathPatch representing the interactively editable path
         # (vertical profile or horizontal flight track in subclasses).
@@ -407,9 +404,6 @@ class PathInteractor(object):
         # Connect mpl events to handler routines: mouse movements and picks.
         canvas = self.ax.figure.canvas
         canvas.mpl_connect('draw_event', self.draw_callback)
-        canvas.mpl_connect('button_press_event', self.button_press_callback)
-        canvas.mpl_connect('button_release_event', self.button_release_callback)
-        canvas.mpl_connect('motion_notify_event', self.motion_notify_callback)
         self.canvas = canvas
 
         # Set the waypoints model, connect to the change() signals of the model
@@ -531,16 +525,6 @@ class PathInteractor(object):
             t.set_visible(self.showverts and self.label_waypoints)
         self.canvas.draw()
 
-    def set_edit_mode(self, mode):
-        """Set the edit mode to one of [MOVE, INSERT, DELETE].
-        """
-        if mode not in [MOVE, INSERT, DELETE]:
-            return
-        self.editmode = mode
-
-    def get_edit_mode(self):
-        return self.editmode
-
     def redraw_path(self, vertices=None):
         """Redraw the matplotlib artists that represent the flight track
            (path patch and line).
@@ -657,8 +641,8 @@ class VPathInteractor(PathInteractor):
         self.numintpoints = numintpoints
         self.redraw_xaxis = redraw_xaxis
         self.clear_figure = clear_figure
-        PathInteractor.__init__(
-            self, ax=ax, waypoints=waypoints, mplpath=PathV([[0, 0]], numintpoints=numintpoints))
+        super(VPathInteractor, self).__init__(
+            ax=ax, waypoints=waypoints, mplpath=PathV([[0, 0]], numintpoints=numintpoints))
 
     def get_num_interpolation_points(self):
         return self.numintpoints
@@ -677,24 +661,30 @@ class VPathInteractor(PathInteractor):
             self.redraw_xaxis(self.path.ilats, self.path.ilons, self.path.itimes)
         self.ax.figure.canvas.draw()
 
-    def button_release_callback(self, event):
+    def button_release_delete_callback(self, event):
         """Called whenever a mouse button is released.
         """
-        if not self.showverts:
-            return
-        if event.button != 1:
+        if not self.showverts or event.button != 1:
             return
 
-        if self.editmode == DELETE and self._ind is not None:
+        if self._ind is not None:
             if self.confirm_delete_waypoint(self._ind):
                 # removeRows() will trigger a signal that will redraw the path.
                 self.waypoints_model.removeRows(self._ind)
+            self._ind = None
 
-        elif self.editmode == INSERT:
-            # TODO: Inserting points is currently not available for side view.
-            pass
+    def button_release_insert_callback(self, event):
+        """Called whenever a mouse button is released.
+        """
+        self._ind = None
 
-        elif self.editmode == MOVE and self._ind is not None:
+    def button_release_move_callback(self, event):
+        """Called whenever a mouse button is released.
+        """
+        if not self.showverts or event.button != 1:
+            return
+
+        if self._ind is not None:
             # Submit the new pressure (the only value that can be edited
             # in the side view) to the data model.
             vertices = self.pathpatch.get_path().vertices
@@ -715,15 +705,7 @@ class VPathInteractor(PathInteractor):
         Hence, points can only be moved in the vertical direction (y position
         in this view).
         """
-        if not self.showverts:
-            return
-        if not self.editmode == MOVE:
-            return
-        if self._ind is None:
-            return
-        if event.inaxes is None:
-            return
-        if event.button != 1:
+        if not self.showverts or self._ind is None or event.inaxes is None or event.button != 1:
             return
         vertices = self.pathpatch.get_path().vertices
         # Set the new y position of the vertex to event.ydata. Keep the
@@ -775,15 +757,14 @@ class HPathInteractor(PathInteractor):
         self.tangent_lines = None
         self.show_tangent_points = False
         self.solar_lines = None
-        self.show_solar_angle = False
+        self.show_solar_angle = None
         self.remote_sensing = None
-        PathInteractor.__init__(self, ax=mplmap.ax, waypoints=waypoints,
-                                mplpath=PathH_GC([[0, 0]], map=mplmap),
-                                facecolor='none', edgecolor='none',
-                                linecolor=linecolor,
-                                markerfacecolor=markerfacecolor,
-                                marker='',
-                                label_waypoints=label_waypoints)
+        super(HPathInteractor, self).__init__(
+            ax=mplmap.ax, waypoints=waypoints,
+            mplpath=PathH_GC([[0, 0]], map=mplmap),
+            facecolor='none', edgecolor='none', linecolor=linecolor,
+            markerfacecolor=markerfacecolor, marker='',
+            label_waypoints=label_waypoints)
 
     def appropriate_epsilon(self, px=5):
         """Determine an epsilon value appropriate for the current projection and
@@ -822,65 +803,76 @@ class HPathInteractor(PathInteractor):
         km_per_px = map_delta / diagonal
         return km_per_px * px
 
-    def button_release_callback(self, event):
+    def button_release_insert_callback(self, event):
         """Called whenever a mouse button is released.
         """
-        if not self.showverts:
-            return
-        if event.button != 1:
+        if not self.showverts or event.button != 1 or event.inaxes is None:
             return
 
-        if self.editmode == DELETE and self._ind is not None:
-            if self.confirm_delete_waypoint(self._ind):
-                # removeRows() will trigger a signal that will redraw the path.
-                self.waypoints_model.removeRows(self._ind)
+        # Get position for new vertex.
+        x, y = event.xdata, event.ydata
+        best_index = self.pathpatch.get_path().index_of_closest_segment(
+            x, y, eps=self.appropriate_epsilon())
+        logging.debug(u"TopView insert point: clicked at (%f, %f), "
+                      u"best index: %d", x, y, best_index)
+        self.pathpatch.get_path().insert_vertex(best_index, [x, y], WaypointsPath.LINETO)
 
-        elif self.editmode == INSERT and event.inaxes is not None:
-            # Get position for new vertex.
-            x, y = event.xdata, event.ydata
-            best_index = self.pathpatch.get_path().index_of_closest_segment(
-                x, y, eps=self.appropriate_epsilon())
-            logging.debug(u"TopView insert point: clicked at (%f, %f), "
-                          u"best index: %d", x, y, best_index)
-            self.pathpatch.get_path().insert_vertex(best_index, [x, y], WaypointsPath.LINETO)
+        lon, lat = self.map(x, y, inverse=True)
+        loc = find_location(lat, lon, tolerance=self.appropriate_epsilon_km(px=15))
+        if loc is not None:
+            (lat, lon), location = loc
+        else:
+            lat, lon = float(round(lat, 2)), float(round(lon, 2))
+            location = u""
+        wpm = self.waypoints_model
+        if len(wpm.all_waypoint_data()) > 0 and 0 < best_index <= len(wpm.all_waypoint_data()):
+            flightlevel = wpm.waypoint_data(best_index - 1).flightlevel
+        elif len(wpm.all_waypoint_data()) > 0 and best_index == 0:
+            flightlevel = wpm.waypoint_data(0).flightlevel
+        else:
+            logging.error(u"Cannot copy flightlevel. best_index: %s, len: %s",
+                          best_index, len(wpm.all_waypoint_data()))
+            flightlevel = 0
+        new_wp = ft.Waypoint(lat, lon, flightlevel, location=location)
+        wpm.insertRows(best_index, rows=1, waypoints=[new_wp])
+        self.redraw_path()
 
-            lon, lat = self.map(x, y, inverse=True)
-            loc = find_location(lat, lon, tolerance=self.appropriate_epsilon_km(px=15))
-            if loc is not None:
-                (lat, lon), location = loc
-            else:
-                lat, lon = round(lat, 2), round(lon, 2)
-                location = u""
-            wpm = self.waypoints_model
-            if len(wpm.all_waypoint_data()) > 0 and 0 < best_index <= len(wpm.all_waypoint_data()):
-                flightlevel = wpm.waypoint_data(best_index - 1).flightlevel
-            elif len(wpm.all_waypoint_data()) > 0 and best_index == 0:
-                flightlevel = wpm.waypoint_data(0).flightlevel
-            else:
-                logging.error(u"Cannot copy flightlevel. best_index: %s, len: %s",
-                              best_index, len(wpm.all_waypoint_data()))
-                flightlevel = 0
-            new_wp = ft.Waypoint(lat, lon, flightlevel, location=location)
-            wpm.insertRows(best_index, rows=1, waypoints=[new_wp])
-            self.redraw_path()
+        self._ind = None
 
-        elif self.editmode == MOVE and self._ind is not None:
-            # Submit the new position to the data model.
-            vertices = self.pathpatch.get_path().wp_vertices
-            lon, lat = self.map(vertices[self._ind][0], vertices[self._ind][1],
-                                inverse=True)
-            loc = find_location(lat, lon, tolerance=self.appropriate_epsilon_km(px=15))
-            if loc is not None:
-                lat, lon = loc[0]
-            else:
-                lat, lon = round(lat, 2), round(lon, 2)
-            # http://doc.trolltech.com/4.3/qabstractitemmodel.html#createIndex
-            # TODO: can lat/lon be submitted together to avoid emitting dataChanged() signals
-            #      twice?
-            qt_index = self.waypoints_model.createIndex(self._ind, ft.LAT)
-            self.waypoints_model.setData(qt_index, QtCore.QVariant(lat))
-            qt_index = self.waypoints_model.createIndex(self._ind, ft.LON)
-            self.waypoints_model.setData(qt_index, QtCore.QVariant(lon))
+    def button_release_move_callback(self, event):
+        """Called whenever a mouse button is released.
+        """
+        if not self.showverts or event.button != 1 or self._ind is None:
+            return
+
+        # Submit the new position to the data model.
+        vertices = self.pathpatch.get_path().wp_vertices
+        lon, lat = self.map(vertices[self._ind][0], vertices[self._ind][1],
+                            inverse=True)
+        loc = find_location(lat, lon, tolerance=self.appropriate_epsilon_km(px=15))
+        if loc is not None:
+            lat, lon = loc[0]
+        else:
+            lat, lon = float(round(lat, 2)), float(round(lon, 2))
+        # http://doc.trolltech.com/4.3/qabstractitemmodel.html#createIndex
+        # TODO: can lat/lon be submitted together to avoid emitting dataChanged() signals
+        #      twice?
+        qt_index = self.waypoints_model.createIndex(self._ind, ft.LAT)
+        self.waypoints_model.setData(qt_index, QtCore.QVariant(lat))
+        qt_index = self.waypoints_model.createIndex(self._ind, ft.LON)
+        self.waypoints_model.setData(qt_index, QtCore.QVariant(lon))
+
+        self._ind = None
+
+    def button_release_delete_callback(self, event):
+        """Called whenever a mouse button is released.
+        """
+        if not self.showverts or event.button != 1:
+            return
+
+        if self._ind is not None and self.confirm_delete_waypoint(self._ind):
+            # removeRows() will trigger a signal that will redraw the path.
+            self.waypoints_model.removeRows(self._ind)
 
         self._ind = None
 
@@ -890,8 +882,6 @@ class HPathInteractor(PathInteractor):
         """
         if not self.showverts:
             return
-        if not self.editmode == MOVE:
-            return
         if self._ind is None:
             return
         if event.inaxes is None:
@@ -960,10 +950,10 @@ class HPathInteractor(PathInteractor):
 
         if self.solar_lines is not None:
             self.solar_lines.remove()
-        if self.show_solar_angle:
+        if self.show_solar_angle is not None:
             assert self.remote_sensing is not None
             self.solar_lines = self.remote_sensing.compute_solar_lines(
-                self.map, wp_vertices, wp_heights, wp_times)
+                self.map, wp_vertices, wp_heights, wp_times, self.show_solar_angle)
             self.ax.add_collection(self.solar_lines)
         else:
             self.solar_lines = None
@@ -1015,7 +1005,7 @@ class HPathInteractor(PathInteractor):
             self.ax.draw_artist(t)
         if self.show_tangent_points:
             self.ax.draw_artist(self.tangent_lines)
-        if self.show_solar_angle:
+        if self.show_solar_angle is not None:
             self.ax.draw_artist(self.solar_lines)
         self.canvas.blit(self.ax.bbox)
 
