@@ -9,7 +9,7 @@
     This file is part of mss.
 
     :copyright: Copyright 2017 Joern Ungermann
-    :copyright: Copyright 2017 by the mss team, see AUTHORS.
+    :copyright: Copyright 2017-2018 by the mss team, see AUTHORS.
     :license: APACHE-2.0, see LICENSE for details.
 
     Licensed under the Apache License, Version 2.0 (the "License");
@@ -27,114 +27,20 @@
 
 from __future__ import division
 
-
 import numpy as np
+from mslib.msui.constants import MSS_CONFIG_PATH
 from mslib.msui.mss_qt import QtGui, QtWidgets
 from mslib.msui.mss_qt import ui_remotesensing_dockwidget as ui
-from mslib.utils import datetime_to_jsec, get_distance, rotate_point, fix_angle, compute_hour_of_day
+from mslib.utils import jsec_to_datetime, datetime_to_jsec, get_distance, rotate_point, fix_angle
 from matplotlib.collections import LineCollection
 from matplotlib.colors import BoundaryNorm, ListedColormap
 import collections
+from skyfield.api import Loader, Topos, utc
 
 
 EARTH_RADIUS = 6371.
 
 
-def compute_view_angles(lon0, lat0, h0, lon1, lat1, h1, angle):
-    mlat = ((lat0 + lat1) / 2.)
-    lon0 *= np.cos(np.deg2rad(mlat))
-    lon1 *= np.cos(np.deg2rad(mlat))
-    dlon = lon1 - lon0
-    dlat = lat1 - lat0
-    obs_azi2 = fix_angle(angle + np.rad2deg(np.arctan2(dlon, dlat)))
-    return obs_azi2, -1
-
-
-def compute_solar_angle(jsec, lon, lat):
-    # The input to the Astronomer's almanach is the difference between
-    # the Julian date and JD 2451545.0 (noon, 1 January 2000)
-    time = (jsec / (60. * 60. * 24.)) - 0.5
-
-    # Mean longitude
-    mnlong = 280.460 + .9856474 * time
-    mnlong %= 360.
-    if mnlong < 0:
-        mnlong += 360
-        assert mnlong >= 0
-
-    # Mean anomaly
-    mnanom = 357.528 + .9856003 * time
-    mnanom = np.deg2rad(mnanom % 360.)
-    if mnanom < 0:
-        mnanom += 2 * np.pi
-        assert mnanom >= 0
-
-    # Ecliptic longitude and obliquity of ecliptic
-    eclong = mnlong + 1.915 * np.sin(mnanom) + 0.020 * np.sin(2 * mnanom)
-    eclong = np.deg2rad(eclong % 360.)
-    if (eclong < 0):
-        eclong += 2 * np.pi
-        assert (eclong >= 0)
-
-    oblqec = np.deg2rad(23.439 - 0.0000004 * time)
-
-    # Celestial coordinates
-    # Right ascension and declination
-    num = np.cos(oblqec) * np.sin(eclong)
-    den = np.cos(eclong)
-    ra = np.arctan(num / den)
-    if den < 0:
-        ra += np.pi
-    elif den >= 0 and num < 0:
-        ra += 2 * np.pi
-
-    dec = np.arcsin(np.sin(oblqec) * np.sin(eclong))
-    # Local coordinates
-    # Greenwich mean sidereal time
-    gmst = 6.697375 + .0657098242 * time + compute_hour_of_day(jsec)
-
-    gmst = gmst % 24.
-    if gmst < 0:
-        gmst += 24
-        assert gmst >= 0
-
-    # Local mean sidereal time
-    if lon < 0:
-        lon += 360
-        assert 0 <= lon <= 360
-
-    lmst = gmst + (lon / 15.)
-    lmst = np.deg2rad(15. * (lmst % 24.))
-
-    # Hour angle
-    ha = lmst - ra
-    if ha < -np.pi:
-        ha += 2 * np.pi
-
-    if ha > np.pi:
-        ha -= 2 * np.pi
-
-    assert -np.pi < ha < 2 * np.pi
-
-    # Latitude to radians
-    lat = np.deg2rad(lat)
-
-    # Azimuth and elevation
-    zenithAngle = np.arccos(np.sin(lat) * np.sin(dec) + np.cos(lat) * np.cos(dec) * np.cos(ha))
-    azimuthAngle = np.arccos(((np.sin(lat) * np.cos(zenithAngle) - np.sin(dec)) /
-                             (np.cos(lat) * np.sin(zenithAngle))))
-
-    if ha > 0:
-        azimuthAngle += np.pi
-    else:
-        azimuthAngle = 3 * np.pi - azimuthAngle % (2 * np.pi)
-
-    if azimuthAngle > np.pi:
-        azimuthAngle -= 2 * np.pi
-
-    return np.rad2deg(azimuthAngle), 90 - np.rad2deg(zenithAngle)
-
-
 class RemoteSensingControlWidget(QtWidgets.QWidget, ui.Ui_RemoteSensingDockWidget):
     """This class implements the remote sensing functionality as dockable widget.
     """
@@ -149,6 +55,9 @@ class RemoteSensingControlWidget(QtWidgets.QWidget, ui.Ui_RemoteSensingDockWidge
         self.setupUi(self)
 
         self.view = view
+        self.load = Loader(MSS_CONFIG_PATH)
+        self.planets = self.load('de421.bsp')
+        self.timescale = self.load.timescale()
 
         button = self.btTangentsColour
         palette = QtGui.QPalette(button.palette())
@@ -158,30 +67,52 @@ class RemoteSensingControlWidget(QtWidgets.QWidget, ui.Ui_RemoteSensingDockWidge
         button.setPalette(palette)
 
         self.dsbTangentHeight.setValue(10.)
-        self.dsbObsAngle.setValue(90.)
+        self.dsbObsAngleAzimuth.setValue(90.)
+        self.dsbObsAngleElevation.setValue(-1.0)
 
         # update plot on every value change
         self.cbDrawTangents.stateChanged.connect(self.update_settings)
         self.cbShowSolarAngle.stateChanged.connect(self.update_settings)
         self.btTangentsColour.clicked.connect(self.set_tangentpoint_colour)
         self.dsbTangentHeight.valueChanged.connect(self.update_settings)
-        self.dsbObsAngle.valueChanged.connect(self.update_settings)
-
+        self.dsbObsAngleAzimuth.valueChanged.connect(self.update_settings)
+        self.dsbObsAngleElevation.valueChanged.connect(self.update_settings)
+        self.cbSolarBody.currentIndexChanged.connect(self.update_settings)
+        self.cbSolarAngleType.currentIndexChanged.connect(self.update_settings)
+        self.lbSolarCmap.setText(
+            "Solar angle colours, dark to light: reds (0-15), violets (15-45), greens (45-180)")
         self.solar_cmap = ListedColormap([
-            (1.0, 0.0, 0.0, 1.0),
-            (1.0, 0.44823529411764707, 0.0, 1.0),
-            (1.0, 0.75, 0.0, 1.0),
-            (0.46999999999999997, 0.10000000000000001, 1.0, 1.0),
-            (0.64666666666666672, 0.25, 1.0, 1.0),
-            (0.82333333333333336, 0.40000000000000002, 1.0, 1.0),
-            (1.0, 0.55000000000000004, 1.0, 1.0),
-            (0.65000000000000002, 1.0, 0.65000000000000002, 1.0),
-            (0.32372549019607844, 0.84941176470588231, 0.32372549019607844, 1.0),
-            (0.0, 0.69999999999999996, 0.0, 1.0)])
-        self.solar_norm = BoundaryNorm([0, 5, 10, 15, 25, 35, 45, 60, 90, 135, 180], self.solar_cmap.N)
+            (1.00, 0.00, 0.00, 1.0),
+            (1.00, 0.45, 0.00, 1.0),
+            (1.00, 0.75, 0.00, 1.0),
+            (0.47, 0.10, 1.00, 1.0),
+            (0.72, 0.38, 1.00, 1.0),
+            (1.00, 0.55, 1.00, 1.0),
+            (0.00, 0.70, 0.00, 1.0),
+            (0.33, 0.85, 0.33, 1.0),
+            (0.65, 1.00, 0.65, 1.0)])
+        self.solar_norm = BoundaryNorm(
+            [0, 5, 10, 15, 25, 35, 45, 90, 135, 180], self.solar_cmap.N)
 
         self.update_settings()
 
+    @staticmethod
+    def compute_view_angles(lon0, lat0, h0, lon1, lat1, h1, obs_azi, obs_ele):
+        mlat = ((lat0 + lat1) / 2.)
+        lon0 *= np.cos(np.deg2rad(mlat))
+        lon1 *= np.cos(np.deg2rad(mlat))
+        dlon = lon1 - lon0
+        dlat = lat1 - lat0
+        obs_azi_p = fix_angle(obs_azi + np.rad2deg(np.arctan2(dlon, dlat)))
+        return obs_azi_p, obs_ele
+
+    def compute_body_angle(self, body, jsec, lon, lat):
+        t = self.timescale.utc(utc.localize(jsec_to_datetime(jsec)))
+        loc = self.planets["earth"] + Topos(lat, lon)
+        astrometric = loc.at(t).observe(self.planets[body])
+        alt, az, d = astrometric.apparent().altaz()
+        return az.degrees, alt.degrees
+
     def update_settings(self):
         """
         Updates settings in TopView and triggers a redraw.
@@ -189,8 +120,12 @@ class RemoteSensingControlWidget(QtWidgets.QWidget, ui.Ui_RemoteSensingDockWidge
         settings = {
             "reference": self,
             "draw_tangents": self.cbDrawTangents.isChecked(),
-            "show_solar_angle": self.cbShowSolarAngle.isChecked(),
         }
+        if self.cbShowSolarAngle.isChecked():
+            settings["show_solar_angle"] = self.cbSolarAngleType.currentText(), self.cbSolarBody.currentText()
+        else:
+            settings["show_solar_angle"] = None
+
         self.view.set_remote_sensing_appearance(settings)
 
     def set_tangentpoint_colour(self):
@@ -228,16 +163,18 @@ class RemoteSensingControlWidget(QtWidgets.QWidget, ui.Ui_RemoteSensingDockWidge
         tplines = [self.tangent_point_coordinates(
             fine_lines[i][0], fine_lines[i][1], line_heights[i],
             cut_height=self.dsbTangentHeight.value()) for i in range(len(fine_lines))]
-        for i, line in enumerate(tplines):
+        dirlines = self.direction_coordinates(wp_lons, wp_lats)
+        lines = tplines + dirlines
+        for i, line in enumerate(lines):
             for j, (lon, lat) in enumerate(line):
                 line[j] = bmap(lon, lat)
-            tplines[i] = line
+            lines[i] = line
         return LineCollection(
-            tplines,
+            lines,
             colors=QtGui.QPalette(self.btTangentsColour.palette()).color(QtGui.QPalette.Button).getRgbF(),
-            zorder=2, animated=True, linewidth=3, linestyles=':')
+            zorder=2, animated=True, linewidth=3, linestyles=[':'] * len(tplines) + ['-'] * len(dirlines))
 
-    def compute_solar_lines(self, bmap, wp_vertices, wp_heights, wp_times):
+    def compute_solar_lines(self, bmap, wp_vertices, wp_heights, wp_times, solartype):
         """
         Computes coloured overlay over the flight path that indicates
         the danger of looking into the sun with a limb sounder aboard
@@ -253,6 +190,8 @@ class RemoteSensingControlWidget(QtWidgets.QWidget, ui.Ui_RemoteSensingDockWidge
                  angle
         """
         # calculate distances and times
+        body, difftype = solartype
+
         times = [datetime_to_jsec(_wp_time) for _wp_time in wp_times]
         x, y = list(zip(*wp_vertices))
         wp_lons, wp_lats = bmap(x, y, inverse=True)
@@ -291,14 +230,16 @@ class RemoteSensingControlWidget(QtWidgets.QWidget, ui.Ui_RemoteSensingDockWidge
         vals = []
         for i in range(len(points) - 1):
             p0, p1 = points[i][0], points[i + 1][0]
-            sol_azi, sol_ele = compute_solar_angle(times[i], p0[0], p0[1])
+
+            sol_azi, sol_ele = self.compute_body_angle(body, times[i], p0[0], p0[1])
+            obs_azi, obs_ele = self.compute_view_angles(
+                p0[0], p0[1], heights[i], p1[0], p1[1], heights[i + 1],
+                self.dsbObsAngleAzimuth.value(), self.dsbObsAngleElevation.value())
             if sol_azi < 0:
                 sol_azi += 360
-            obs_azi, obs_ele = compute_view_angles(p0[0], p0[1], heights[i], p1[0], p1[1], heights[i + 1],
-                                                   self.dsbObsAngle.value())
             if obs_azi < 0:
                 obs_azi += 360
-            rating = self.calc_view_rating(obs_azi, obs_ele, sol_azi, sol_ele, heights[i])
+            rating = self.calc_view_rating(obs_azi, obs_ele, sol_azi, sol_ele, heights[i], difftype)
             vals.append(rating)
 
         # convert lon, lat to map points
@@ -324,12 +265,14 @@ class RemoteSensingControlWidget(QtWidgets.QWidget, ui.Ui_RemoteSensingDockWidge
         Returns: List of tuples of longitude/latitude coordinates
 
         """
-        lon_lin2 = np.array(lon_lin) * np.cos(np.deg2rad(np.array(lat_lin)))
-        lins = list(zip(lon_lin2[0:-1], lon_lin2[1:], lat_lin[0:-1], lat_lin[1:]))
+        lins = list(zip(lon_lin[0:-1], lon_lin[1:], lat_lin[0:-1], lat_lin[1:]))
+        lins = [(x0 * np.cos(np.deg2rad(np.mean([y0, y1]))), x1 * np.cos(np.deg2rad(np.mean([y0, y1]))), y0, y1)
+                for x0, x1, y0, y1 in lins]
+
         direction = [(x1 - x0, y1 - y0) for x0, x1, y0, y1 in lins]
         direction = [(_x / np.hypot(_x, _y), _y / np.hypot(_x, _y))
                      for _x, _y in direction]
-        los = [rotate_point(point, -self.dsbObsAngle.value()) for point in direction]
+        los = [rotate_point(point, -self.dsbObsAngleAzimuth.value()) for point in direction]
         los.append(los[-1])
 
         if isinstance(flight_alt, (collections.Sequence, np.ndarray)):
@@ -346,8 +289,41 @@ class RemoteSensingControlWidget(QtWidgets.QWidget, ui.Ui_RemoteSensingDockWidge
         tps = [(x0 / np.cos(np.deg2rad(y0)), y0) for (x0, y0) in tps]
         return tps
 
+    def direction_coordinates(self, lon_lin, lat_lin):
+        """
+        Computes coordinates of tangent points given coordinates of flight path.
+
+        Args:
+            lon_lin: longitudes of flight path
+            lat_lin: latitudes of flight path
+            flight_alt: altitude of aircraft (scalar or numpy array)
+            cut_height: altitude of tangent points
+
+        Returns: List of tuples of longitude/latitude coordinates
+
+        """
+        lins = list(zip(lon_lin[0:-1], lon_lin[1:], lat_lin[0:-1], lat_lin[1:]))
+        lins = [(x0 * np.cos(np.deg2rad(np.mean([y0, y1]))), x1 * np.cos(np.deg2rad(np.mean([y0, y1]))), y0, y1)
+                for x0, x1, y0, y1 in lins]
+        lens = [np.hypot(x1 - x0, y1 - y0) * 110. for x0, x1, y0, y1 in lins]
+        lins = [_x for _x, _l in zip(lins, lens) if _l > 10]
+
+        direction = [(0.5 * (x0 + x1), 0.5 * (y0 + y1), x1 - x0, y1 - y0) for x0, x1, y0, y1 in lins]
+        direction = [(_u, _v, _x / np.hypot(_x, _y), _y / np.hypot(_x, _y))
+                     for _u, _v, _x, _y in direction]
+        los = [rotate_point(point[2:], -self.dsbObsAngleAzimuth.value()) for point in direction]
+
+        dist = 1.
+
+        tp_dir = (np.array(los).T * dist).T
+
+        tps = [(x0, y0, x0 + tp_x, y0 + tp_y) for
+               ((x0, y0, _, _), (tp_x, tp_y)) in zip(direction, tp_dir)]
+        tps = [[(x0 / np.cos(np.deg2rad(y0)), y0), (x1 / np.cos(np.deg2rad(y0)), y1)] for (x0, y0, x1, y1) in tps]
+        return tps
+
     @staticmethod
-    def calc_view_rating(obs_azi, obs_ele, sol_azi, sol_ele, height):
+    def calc_view_rating(obs_azi, obs_ele, sol_azi, sol_ele, height, difftype):
         """
         Calculates the angular distance between given directions under the
         condition that the sun is above the horizon.
@@ -361,10 +337,16 @@ class RemoteSensingControlWidget(QtWidgets.QWidget, ui.Ui_RemoteSensingDockWidge
 
         Returns: angular distance or 180 degrees if sun is below horizon
         """
-        thresh = -np.rad2deg(np.arccos(EARTH_RADIUS / (height + EARTH_RADIUS))) - 3
-
         delta_azi = obs_azi - sol_azi
-        delta_ele = obs_ele + sol_ele
-        if sol_ele < thresh:
-            delta_ele = 180
-        return np.linalg.norm([delta_azi, delta_ele])
+        delta_ele = obs_ele - sol_ele
+        if "horizon" in difftype:
+            thresh = -np.rad2deg(np.arccos(EARTH_RADIUS / (height + EARTH_RADIUS))) - 3
+            if sol_ele < thresh:
+                delta_ele = 180
+
+        if "azimuth" == difftype:
+            return np.abs(obs_azi - sol_azi)
+        elif "elevation" == difftype:
+            return np.abs(obs_ele - sol_ele)
+        else:
+            return np.hypot(delta_azi, delta_ele)
