@@ -10,7 +10,7 @@
 
     :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
     :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
-    :copyright: Copyright 2016-2017 by the mss team, see AUTHORS.
+    :copyright: Copyright 2016-2018 by the mss team, see AUTHORS.
     :license: APACHE-2.0, see LICENSE for details.
 
     Licensed under the Apache License, Version 2.0 (the "License");
@@ -32,12 +32,14 @@ import os
 
 import mslib.msui.flighttrack as ft
 from mslib import thermolib
+from fs import open_fs
 
 
 def load_from_flitestar(filename):
     waypoints = []
-
-    with open(filename, 'r') as f:
+    _dirname, _name = os.path.split(filename)
+    _fs = open_fs(_dirname)
+    with _fs.open(_name, 'r') as f:
         firstline = f.readline()
         if not firstline.startswith("# FliteStar/FliteMap generated flight plan."):
             raise SyntaxError("The file does not seem to be a FliteStar file!")
